@@ -20,13 +20,16 @@ from __future__ import print_function
 
 import collections
 import functools
+import inspect
 import math
+import time
 
 from absl import logging
 
 from dopamine.agents.dqn import dqn_agent
 from dopamine.jax import networks
 from dopamine.replay_memory import circular_replay_buffer
+from dopamine.replay_memory import prioritized_replay_buffer
 from flax import nn
 from flax import optim
 import gin
@@ -39,16 +42,39 @@ import tensorflow as tf
 NATURE_DQN_OBSERVATION_SHAPE = dqn_agent.NATURE_DQN_OBSERVATION_SHAPE
 NATURE_DQN_DTYPE = jnp.uint8
 NATURE_DQN_STACK_SIZE = dqn_agent.NATURE_DQN_STACK_SIZE
+identity_epsilon = dqn_agent.identity_epsilon
 
 
 @gin.configurable
 def create_optimizer(name='adam', learning_rate=6.25e-5, beta1=0.9, beta2=0.999,
-                     eps=1.5e-4):
+                     eps=1.5e-4, centered=False):
+  """Create an optimizer for training.
+
+  Currently, only the Adam optimizer is supported.
+
+  Args:
+    name: str, name of the optimizer to create.
+    learning_rate: float, learning rate to use in the optimizer.
+    beta1: float, beta1 parameter for the optimizer.
+    beta2: float, beta2 parameter for the optimizer.
+    eps: float, epsilon parameter for the optimizer.
+    centered: bool, centered parameter for RMSProp.
+
+  Returns:
+    A flax optimizer.
+  """
   if name == 'adam':
+    logging.info('Creating Adam optimizer with settings lr=%f, beta1=%f, '
+                 'beta2=%f, eps=%f', learning_rate, beta1, beta2, eps)
     return optim.Adam(
         learning_rate=learning_rate, beta1=beta1, beta2=beta2, eps=eps)
+  elif name == 'rmsprop':
+    logging.info('Creating RMSProp optimizer with settings lr=%f, beta2=%f, '
+                 'eps=%f', learning_rate, beta2, eps)
+    return optim.RMSProp(learning_rate=learning_rate, beta2=beta2, eps=eps,
+                         centered=centered)
   else:
-    raise ValueError(f'Unknown optimizer {name}')
+    raise ValueError('Unsupported optimizer {}'.format(name))
 
 
 def huber_loss(targets, predictions, delta=1.0):
@@ -95,6 +121,7 @@ def target_q(target_network, next_states, rewards, terminals, cumulative_gamma):
                                (1. - terminals))
 
 
+@gin.configurable
 @functools.partial(jax.jit, static_argnums=(0, 2, 3))
 def linearly_decaying_epsilon(decay_period, step, warmup_steps, epsilon):
   """Returns the current epsilon for the agent's epsilon-greedy policy.
@@ -186,9 +213,15 @@ class JaxDQNAgent(object):
                optimizer='adam',
                summary_writer=None,
                summary_writing_frequency=500,
-               allow_partial_reload=False):
+               allow_partial_reload=False,
+               seed=None):
     """Initializes the agent and constructs the necessary components.
 
+    Note: We are using the Adam optimizer by default for JaxDQN, which differs
+          from the original NatureDQN and the dopamine TensorFlow version. In
+          the experiments we have ran, we have found that using Adam yields
+          improved training performance.
+
     Args:
       num_actions: int, number of actions the agent can take at any state.
       observation_shape: tuple of ints describing the observation shape.
@@ -218,8 +251,11 @@ class JaxDQNAgent(object):
         written. Lower values will result in slower training.
       allow_partial_reload: bool, whether we allow reloading a partial agent
         (for instance, only the network parameters).
+      seed: int, a seed for DQN's internal RNG, used for initialization and
+        sampling actions. If None, will use the current time in nanoseconds.
     """
     assert isinstance(observation_shape, tuple)
+    seed = int(time.time() * 1e6) if seed is None else seed
     logging.info('Creating %s agent with the following parameters:',
                  self.__class__.__name__)
     logging.info('\t gamma: %f', gamma)
@@ -233,6 +269,7 @@ class JaxDQNAgent(object):
     logging.info('\t optimizer: %s', optimizer)
     logging.info('\t max_tf_checkpoints_to_keep: %d',
                  max_tf_checkpoints_to_keep)
+    logging.info('\t seed: %d', seed)
 
     self.num_actions = num_actions
     self.observation_shape = tuple(observation_shape)
@@ -255,7 +292,7 @@ class JaxDQNAgent(object):
     self.summary_writing_frequency = summary_writing_frequency
     self.allow_partial_reload = allow_partial_reload
 
-    self._rng = jax.random.PRNGKey(0)
+    self._rng = jax.random.PRNGKey(seed)
     state_shape = self.observation_shape + (stack_size,)
     self.state = onp.zeros(state_shape)
     self._replay = self._build_replay_buffer()
@@ -397,7 +434,7 @@ class JaxDQNAgent(object):
     self.action = onp.asarray(self.action)
     return self.action
 
-  def end_episode(self, reward):
+  def end_episode(self, reward, terminal=True):
     """Signals the end of the episode to the agent.
 
     We store the observation of the current time step, which is the last
@@ -405,9 +442,17 @@ class JaxDQNAgent(object):
 
     Args:
       reward: float, the last reward from the environment.
+      terminal: bool, whether the last state-action led to a terminal state.
     """
     if not self.eval_mode:
-      self._store_transition(self._observation, self.action, reward, True)
+      argspec = inspect.getfullargspec(self._store_transition)
+      if 'episode_end' in argspec.args or 'episode_end' in argspec.kwonlyargs:
+        self._store_transition(
+            self._observation, self.action, reward, terminal, episode_end=True)
+      else:
+        logging.warning(
+            '_store_transition function doesn\'t have episode_end arg.')
+        self._store_transition(self._observation, self.action, reward, terminal)
 
   def _train_step(self):
     """Runs a single training step.
@@ -443,19 +488,52 @@ class JaxDQNAgent(object):
 
     self.training_steps += 1
 
-  def _store_transition(self, last_observation, action, reward, is_terminal):
-    """Stores an experienced transition.
+  def _store_transition(self,
+                        last_observation,
+                        action,
+                        reward,
+                        is_terminal,
+                        *args,
+                        priority=None,
+                        episode_end=False):
+    """Stores a transition when in training mode.
 
-    Pedantically speaking, this does not actually store an entire transition
-    since the next state is recorded on the following time step.
+    Stores the following tuple in the replay buffer (last_observation, action,
+    reward, is_terminal, priority).
 
     Args:
-      last_observation: numpy array, last observation.
-      action: int, the action taken.
-      reward: float, the reward.
-      is_terminal: bool, indicating if the current state is a terminal state.
+      last_observation: Last observation, type determined via observation_type
+        parameter in the replay_memory constructor.
+      action: An integer, the action taken.
+      reward: A float, the reward.
+      is_terminal: Boolean indicating if the current state is a terminal state.
+      *args: Any, other items to be added to the replay buffer.
+      priority: Float. Priority of sampling the transition. If None, the default
+        priority will be used. If replay scheme is uniform, the default priority
+        is 1. If the replay scheme is prioritized, the default priority is the
+        maximum ever seen [Schaul et al., 2015].
+      episode_end: bool, whether this transition is the last for the episode.
+        This can be different than terminal when ending the episode because
+        of a timeout, for example.
     """
-    self._replay.add(last_observation, action, reward, is_terminal)
+    is_prioritized = isinstance(
+        self._replay,
+        prioritized_replay_buffer.OutOfGraphPrioritizedReplayBuffer)
+    if is_prioritized and priority is None:
+      if self._replay_scheme == 'uniform':
+        priority = 1.
+      else:
+        priority = self._replay.sum_tree.max_recorded_priority
+
+    if not self.eval_mode:
+      self._replay.add(
+          last_observation,
+          action,
+          reward,
+          is_terminal,
+          *args,
+          priority=priority,
+          episode_end=episode_end)
 
   def bundle_and_checkpoint(self, checkpoint_dir, iteration_number):
     """Returns a self-contained bundle of the agent's state.
